local Postmaster = { -- List of sender names to delete mail from
 	-- Lost item guy
	['Thaumaturge Vashreen'] = true, -- English
	
	-- The Postmaster
	['The Postmaster'] = true, -- English
	['Der Postmeister'] = true, -- German
	['El Jefe de correos'] = true, -- Spanish
	['Le maître de poste'] = true, -- French
	['Il Postino'] = true, -- Italian
	['O Chefe do Correio'] = true, -- Portuguese
	['?????????'] = true, -- Russian
}

local UniqueCache = {} -- store items so we don't have to re-check their tooltips
local ITEM_UNIQUE_MULTIPLE, ITEM_UNIQUE = ITEM_UNIQUE_MULTIPLE:gsub('%(%%d%)', '%%((%d+)%%)'), ITEM_UNIQUE
local ScanTip = CreateFrame('GameTooltip', 'PostmasterGeneralTooltip', nil, 'GameTooltipTemplate')
local function GetUnique(itemLink) -- returns the value of the second line of the tooltip
	if itemLink then
		if UniqueCache[itemLink] then -- return stored result
			return UniqueCache[itemLink]
		end
		ScanTip:SetOwner(UIParent, 'ANCHOR_NONE')
		ScanTip:SetHyperlink(itemLink)
		local numLines = ScanTip:NumLines()
		for i = 1, numLines do
			local line = _G['PostmasterGeneralTooltipTextLeft' .. i]
			if line then
				local text = line:GetText() or ''
				local quantity = text:match(ITEM_UNIQUE_MULTIPLE)
				if text == ITEM_UNIQUE or quantity then
					UniqueCache[itemLink] = (quantity or 1) + 0
					return UniqueCache[itemLink]
				end
			end
		end
		if numLines > 0 then
			UniqueCache[itemLink] = false
		end
	end
end

local function HasSpaceFor(itemLink, itemCount) -- check if there are any free slots or less-than-complete stacks of an item in your inventory
	if GetUnique(itemLink) then return false end -- bail if item is at all unique
	
	if not itemCount then itemCount = 1 end
	local stackSize = select(8, GetItemInfo(itemLink))
	if itemCount < stackSize then -- look for partial stacks of the same item
		for bag = BACKPACK_CONTAINER, NUM_BAG_SLOTS do -- open every lootable container in your inventory
			for slot = 1, GetContainerNumSlots(bag) do
				local texture, containerCount, locked, quality, readable, lootable, containerLink = GetContainerItemInfo(bag, slot)
				if not locked and containerLink == itemLink and containerCount + itemCount <= stackSize then
					return true
				end
			end
		end
	end
	
	local itemFamily = GetItemFamily(itemLink)
	for i = BACKPACK_CONTAINER, NUM_BAG_SLOTS do
		local freeSlots, bagFamily = GetContainerNumFreeSlots(i)
		if (bagFamily == 0 or ( itemFamily and bit.band(itemFamily, bagFamily) > 0 )) and freeSlots > 0 then
			return true
		end
	end
	return false
end

local f = CreateFrame('Frame')
f:Hide()

local timesince = 0
f:SetScript('OnUpdate', function(self, elapsed)
	timesince = timesince + elapsed
	if timesince >= 0.1 then -- scan while mailframe is open
		for i = GetInboxNumItems(), 1, -1 do
			local packageIcon, stationeryIcon, sender, subject, money, CODAmount, daysLeft, hasItem, wasRead, wasReturned, textCreated, canReply, isGM = GetInboxHeaderInfo(i)
			if Postmaster[sender] then -- AutoLootMailItem(i) -- same as shift-clicking the mail icon
				if money and money > 0 then
					TakeInboxMoney(i)
				end
				
				if hasItem then
					for j = 1, ATTACHMENTS_MAX_RECEIVE do
						local name, texture, count, quality, canUse = GetInboxItem(i, j)
						local itemLink = GetInboxItemLink(i, j)
						if itemLink and HasSpaceFor(itemLink, count) then
							TakeInboxItem(i, j)
						end
					end
				elseif InboxItemCanDelete(i) then
					DeleteInboxItem(i)
				end
			end
		end
		timesince = 0
	end
end)

f:SetScript('OnEvent', function(self, event, ...)
	self:SetShown(event == 'MAIL_SHOW')
end)
f:RegisterEvent('MAIL_SHOW')
f:RegisterEvent('MAIL_CLOSED')